/*
**
**	palmstd_lib.c
**
**	PalmOS ANSI Libs compatible functions
**
**
*/
#include "palmstd_lib.h"
#include "vsprintf.h"
	


/** 
 *
 * Incomplete self-test of standard lib.
 * Only tests the conversion routines on sprintf
 *
 */
void stdlib_self_test(void) {
	static char s[500];
	
	sprintf(s, "string(s):('teste'='%s')\n", "teste");
	puts(s);
	
	sprintf(s, "char(c):('\"=%c'),('%%=%c'),('\\=%c')\n", '\"', '%', '\\');
	puts(s);
	
	sprintf(s, "int(d):(0=%d),(-32767=%d),(32767=%d)\n", 0, -32767, 32767);
	puts(s);
	
	sprintf(s, "uint(u):(0U=%u),(65535U=%u)\n", 0U, 65535U);
	puts(s);
	
	sprintf(s, "float(f):(0=%f),(1=%f),(1.5=%f),(-1.5=%f)\n", 0.0F, 1.0F, 1.5F, -1.5F);
	puts(s);
	
	sprintf(s, "double(g):(0=%g),(1=%g),(1.5=%g),(-1.5=%g)\n", (double) 0.0, (double) 1.0, (double) 1.5, (double)-1.5);
	puts(s);
}



long strtol(const char *cp,char **endp,unsigned int base)
{
	if(*cp=='-')
		return -strtoul(cp+1,endp,base);
	return strtoul(cp,endp,base);
}


unsigned long strtoul(const char *cp,char **endp,unsigned int base)
{
         unsigned long result = 0,value;
 
         if (!base) {
                 base = 10;
                 if (*cp == '0') {
                         base = 8;
                         cp++;
                         if ((*cp == 'x') && isxdigit(cp[1])) {
                                 cp++;
                                 base = 16;
                         }
                 }
         }
         while (isxdigit(*cp) && (value = isdigit(*cp) ? *cp-'0' : (islower(*cp)
             ? toupper(*cp) : *cp)-'A'+10) < base) {
                 result = result*base + value;
                 cp++;
         }
         if (endp)
                 *endp = (char *)cp;
         return result;
}


/**
 * exit - Equivalent to ANSI exit() function
 *
 * @code: The exit code. 0 means success!
 */
void exit(int code) {
	EventType e;
	
	// S se h cdigo de erro != 0
	ErrNonFatalDisplayIf(code, "Internal error!! Exiting application...");
	
	e.eType = appStopEvent;
	EvtAddEventToQueue(&e);
}


/**
 * atime - Equivalent to ANSI time() function
 *
 * @t: pointer to the time variable that'll receive the current time
 */
//time_t atime(time_t * t) {
//	time_t result = TimGetSeconds();
//	if (t) 
//		*t = result;
//	return result;
//}

/**
 * amktime - alternative to mktime ANSI function
 *
 * @... not tested
 */
//time_t amktime(int tm_year, int tm_mon, int tm_mday, int tm_hour, int tm_min, int tm_sec, char tm_isdst) {
//	DateTimeType dt;
//	dt.second = tm_sec;
//	dt.minute = tm_min;
//	dt.hour   = tm_hour;
//	dt.day    = tm_mday;
//	dt.month  = tm_mon;
//	dt.year   = tm_year;
	
//	return TimDateTimeToSeconds(&dt);
//}







/**
 * strnlen - Find the length of a length-limited string
 *
 * fonte: http://lxr.linux.no/source/lib/string.c
 *
 * @s: The string to be sized
 * @count: The maximum number of bytes to search
 */
size_t strnlen(const char * s, size_t count)
{
	const char *sc;
	for (sc = s; count-- && *sc != '\0'; ++sc) /* proposital nothing */;
	return sc - s;
}



/**
 * strcspn - Return the size of the non-matching string
 *
 * fonte: http://lxr.linux.no/source/lib/string.c
 *
 * @s: String to be searched
 * @reject: String to reject
 */
int strcspn(const char *s, const char *reject)
{
    int count = 0;

    while (*s != '\0') {
		if (strchr(reject, *s++) != NULL) return count;
		count++;
    }
    return count;
}


/** 
 *
 * Helps reallocating a block of memory
 *
 */
void * MemPtrRealloc(void *oldp, unsigned long size)
{
	void *newp;
	if (MemPtrResize(oldp, size) == 0) return oldp;
	
	newp = MemPtrNew(size);
	
	if (!newp) return 0;  // fail to allocate
	
	MemMove(newp, oldp, MemPtrSize(oldp));  // move contents
	MemPtrFree(oldp); 	// free old
	return newp;	// return new
}



/** 
 *
 * Adapted from strToDouble below
 *
 */
double strtod(const char *str, char **end)
{ 
	Int16  i, start, length, punctPos;
	double  result, sign, fractPart;

	result = fractPart = 0.0;
	length = punctPos = StrLen(str);

	start = 0;
	sign = 1.0;
	if (str[0] == '-') {
		sign = -1.0;
		start = 1;
	}

	// parse the string from left to right converting the integer part
	for (i=start; i<length; i++) {
		if (str[i] != '.') {
			if (isdigit(str[i])) {
				result = result * 10.0 + (str[i] - 0x30);
			}
			else { 
				*end = (char *)&(str[i]);
				return result;
			}
		}
		else { 
			punctPos = i;
			break;
		}
	}


	// parse the string from the end to the '.' converting the fractional part
	if (str[punctPos] == '.') {
		for (i=length-1; i>punctPos; i--) {
			if (isdigit(str[i])) {
				fractPart = fractPart / 10.0 + (str[i] - 0x30);
			}
			else { 
				*end = (char *)&(str[i]);
				return result += fractPart / 10.0;
			}
		}
		result += fractPart / 10.0;
	}

	// correcting the sign
	return result * sign;				
}


/** 
 *
 * Adapted from doubleToStr below.
 * Diff: Takes off extra zeroes after the decimal point.
 *
 */
void dtostr(char * str, double flpNumber, Int16 numFractDigits)
{
	Int16 i;
	doubleToStr(str, flpNumber, numFractDigits);
	
	for (i=0; (str[i] != '\0' && str[i] != '.'); i++) ; // proposital
	
	if (str[i] == '.') {
		for (i=strlen(str)-1; (i > 0 && str[i] == '0'); i--) str[i] = '\0';
	}
	if (str[i] == '.') str[i] = '\0'; // se str="123.000", tem que tirar o ponto
}



/**
 * memchr - Find a character in an area of memory.
 *
 * fonte: http://lxr.linux.no/source/lib/string.c
 *
 * @s: The memory area
 * @c: The byte to search for
 * @n: The size of the area.
 *
 * returns the address of the first occurrence of @c, or %NULL
 * if @c is not found
 */
void * memchr(const void *s, int c, size_t n)
{
	const unsigned char *p = s;
	while (n-- != 0) {
		if ((unsigned char)c == *p++) {
			return (void *)(p-1);
		}
	}
	return NULL;
}



/**
 * strpbrk - Find the first occurrence of a set of characters
 *
 * fonte: http://lxr.linux.no/source/lib/string.c
 *
 * @cs: The string to be searched
 * @ct: The characters to search for
 */
char * strpbrk(const char * cs,const char * ct)
{
	const char *sc1,*sc2;

	for( sc1 = cs; *sc1 != '\0'; ++sc1) {
		for( sc2 = ct; *sc2 != '\0'; ++sc2) {
			if (*sc1 == *sc2)
				return (char *) sc1;
		}
	}
	return NULL;
}


/**
 * doubleToStr - Routine to convert double to string in fixed point notation.
 *				This is limited to 9 fractional digits.
 *
 * Fonte:  http://www.40-below.com/palm/dtostr.html  
 *
 * @str:            String buffer
 * @flpNumber:      The number to convert
 * @numFractDigits: Number of decimals do be used
 */
void doubleToStr(char * str, double flpNumber, Int16 numFractDigits)
{ 
	Int32  longNumber;
	double flpIP, zeros, arredondamento;
	Int16 i, remainder, strLen, start;
	Char sign = ' ';					// a ' ' or '-' will be output for more uniform display output

	if (numFractDigits == -1) numFractDigits = 9;
	
	if (numFractDigits < 0 || numFractDigits > 9) {
		return ;
	}

	if (flpNumber < 0.0) {
		flpNumber = -flpNumber;
		sign = '-';
	}

	// This could also be done with an array 1.0, 10.0, 100.0 etc.
	zeros = 1.0;
	for (i=0; i<numFractDigits; i++)
		zeros *= 10;

	arredondamento = 0.5/zeros;
	flpNumber += arredondamento;
	flpIP = (Int32) flpNumber;
	flpNumber = flpNumber - flpIP;

	// Don't add an extra space on the beginning of the string.
    start = (sign == '-' ? 1 : 0);
	if (start) str[0] = sign;
	
	// Convert higher part
	StrIToA(&str[start], (Int32) flpIP);
	
	// put in the decimal point and terminate the string
	strLen = StrLen(str);					
	str[strLen] = '.';
	str[numFractDigits+strLen+1] = '\0';

	// fractional part -- this is what limits to 9 digits
	longNumber = flpNumber * zeros;			

	// convert the integer part
	for (i=numFractDigits+strLen; i>strLen; i--) { 
		remainder = longNumber % 10;
		str[i] = remainder + 0x30;
		longNumber /= 10;
	}
}



/**
 * strToDouble - Routine to convert a string to a double -- Allowed input is in fixed notation ddd.fff
 *
 * Fonte:  http://www.40-below.com/palm/strtod.html
 *
 * @str:            String to be converted
 * @flpNumber:      The number to convert
 * @numFractDigits: Number of decimals do be used
 */
double strToDouble(char * str)
{ 
	Int16  i, start, length, punctPos;
	double  result, sign, fractPart;

	result = fractPart = 0.0;
	length = punctPos = StrLen(str);

	start = 0;
	sign = 1.0;
	if (str[0] == '-') { 
		sign = -1.0;
		start = 1;
	}

	// parse the string from left to right converting the integer part
	for (i=start; i<length; i++)	{
 
		if (str[i] != '.') {
			if (isdigit(str[i])) {
				result = result * 10.0 + (str[i] - 0x30);
			}
			else { 
				return 0.0;
			}
		}
		else { 
			punctPos = i;
			break;
		}
	}


	// parse the string from the end to the '.' converting the fractional part
	if (str[punctPos] == '.') {
		for (i=length-1; i>punctPos; i--) {
			if (isdigit(str[i])) {
				fractPart = fractPart / 10.0 + (str[i] - 0x30);
			}
			else { 
				return 0.0;
			}
		}
		result += fractPart / 10.0;
	}

	// correcting the sign
	return result * sign;				
}


/*
 * FUNCTION: GetObjectPtr
 *
 * DESCRIPTION:
 *
 * This routine returns a pointer to an object in the current form.
 *
 * PARAMETERS:
 *
 * formId
 *     id of the form to display
 *
 * RETURNED:
 *     address of object as a void pointer
 */
extern void * GetObjectPtr(UInt16 objectID)
{
	FormType * frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}





/*
 * Simulao de stdio, para sada de dados na tela.
 *
 */
 
/* Coordenadas na tela */
Int16 y = 15, x = 0;

extern void puts(const char * s) {
	Int16 i, w;
	
	for (i=0; s[i] != '\0'; i++) {
	
		if (s[i] == '\n') {
			y = y + 12;
			x = 0;
			continue;
		}
		w = FntCharWidth(s[i]);
		if (x + w > 160) {
			y = y + 12;
			x = 0;
		}
		
		WinDrawChar(s[i], x, y);
		x = x + w;
	}
}
