/*
**
**	PalmStd_lib.h
**
**	Used on Lua5 to PalmOS Port, by Jos Machado
**
**	This is intendend to deceive the vsprintf and the lua ANSI code.
**
**
**	
**
*/
#ifndef __PALM_STD_LIB__
#define __PALM_STD_LIB__



#include <palmos.h>
#include <stdarg.h>
#include <stddef.h>

unsigned long strtoul(const char *cp,char **endp,unsigned int base);
long strtol(const char *cp,char **endp,unsigned int base);


/* we're kidnapping some system functions... */
#define memmove 		MemMove
#define memcmp  		MemCmp
#define memcpy(d,o,s)	((MemMove(d,o,s) == 0)?d:NULL)
#define memset(p,c,s)	MemSet(p,s,c)


#define strchr(p,c)		StrChr(p,c)
#define strlen(p)		StrLen(p)
#define strcmp(s1,s2)	StrCompare(s1,s2)
#define strcoll(s1,s2)	StrCompare(s1,s2)
#define strcpy			StrCopy
#define strncpy			StrNCopy
#define strcat			StrCat
#define strncat			StrNCat


#define isdigit(c)		(c > 0 ? (TxtCharIsDigit(c)) : 0)
#define isspace(c)		(c > 0 ? (TxtCharIsSpace(c)) : 0)
#define isalpha(c)		(c > 0 ? (TxtCharIsAlpha(c)) : 0)
#define iscntrl(c)		(c > 0 ? (TxtCharIsCntrl(c)) : 0)
#define isalnum(c)		(c > 0 ? (TxtCharIsAlNum(c)) : 0)
#define islower(c)		(c > 0 ? (TxtCharIsLower(c)) : 0)
#define isupper(c)		(c > 0 ? (TxtCharIsUpper(c)) : 0)
#define ispunct(c)		(c > 0 ? (TxtCharIsPunct(c)) : 0)
#define isxdigit(c)		(c > 0 ? (TxtCharIsHex(c))   : 0)
#define tolower(c)		(('A' <= (c) && (c) <= 'Z')	|| (192 <= (c) && (c) <= 223) ? (c) + ('a' - 'A') : (c))
#define toupper(c)		(('a' <= (c) && (c) <= 'z')	|| (224 <= (c) && (c) <= 255) ? (c) + ('A' - 'a') : (c))


#define malloc			MemPtrNew
#define realloc(b,s)	((b) == NULL ? MemPtrNew(s) : MemPtrRealloc(b,s))
#define free(b)			if (b) MemPtrFree(b)
#define calloc(n,s)		MemPtrNew(n*s)


//#define abs(A) 			((A) > 0 ? (A) : -(A)) /* not safe */
#define abs(A) 			fabs(A) /* using mathlib version */


/* and creating new ones as we need... */
#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1
void exit(int code);


/* declared on palmstd_lib.c */
size_t			strnlen			(const char *s, size_t maxlen); 
int 			strcspn			(const char *s, const char *reject);
void * 			MemPtrRealloc	(void *p, unsigned long size);
void * 			memchr			(const void * src, int val, size_t n);
char * 			strpbrk			(const char * str, const char * set);
void 			puts			(const char * s);

void 			doubleToStr(char * str, double flpNumber, Int16 numFractDigits);
double			strToDouble(char * str);
void 			dtostr(char * str, double flpNumber, Int16 numFractDigits);
double			strtod(const char *s, char **end);




// Self test
void stdlib_self_test(void);



/* What's our debug _ALERT function ? */
#include "debug_lib.h"
#define LuaMessage alert


/* Useful PalmOS Functions */
extern void * GetObjectPtr(UInt16 objectID);


#endif
